/*
** Copyright (c) 1996, 2002
**      Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that: (1) Redistributions of
** source code must retain the above copyright notice, this list of
** conditions and the following disclaimer, (2) Redistributions in
** binary form must reproduce the above copyright notice, this list
** of conditions and the following disclaimer in the documentation
** and/or other materials provided with the distribution, (3) All
** advertising materials mentioning features or use of this software
** must display the following acknowledgment: ``This product includes
** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
** The name of the author may not be used to endorse or promote products
** derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

/*
** xnetintf - control a network interface using X-Windows.
*/

const char *Version = "1.8 20020506";

const char *CopyrightVersion = "xnetintf %s\n\nCopyright (c) 1996, 2002 Jeff Forys.  All rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, is permitted under the terms specified in the source\ncode (and manual pages).\n";

#ifndef lint
static char rcsid[] = "$Id: xnetintf.c,v 1.10 2002/05/07 01:39:05 forys Exp $";
#endif

#include <unistd.h>
#include <stdio.h>

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include "Netintf.h"
#include <X11/Xaw/Cardinals.h>

char *ProgName;

static void delwin _P_((Widget w, XEvent *, String *, Cardinal *));

static XrmOptionDescRec options[] = {
	{ "-ifname",  "*netintf.ifname",  XrmoptionSepArg, (caddr_t)NULL },
	{ "-confdir", "*netintf.confdir", XrmoptionSepArg, (caddr_t)NULL },
	{ "-maxival", "*netintf.maxival", XrmoptionSepArg, (caddr_t)NULL },
	{ "-volume",  "*netintf.volume",  XrmoptionSepArg, (caddr_t)NULL },
	{ "-dbglvl",  "*netintf.dbglvl",  XrmoptionSepArg, (caddr_t)NULL },
};

static XtActionsRec xnetintf_acts[] = { { "delwin", delwin } };
static Atom wm_del;
static Widget w_netintf;

int
main(argc, argv)
	int argc;
	char *argv[];
{
	XtAppContext xtcontext;
	Widget dsp;

	if ((ProgName = strrchr(*argv, '/')) != NULL)
		ProgName++;
	else
		ProgName = *argv;

	/* "-v" alone prints version information and exits successfully */
	if (argc == 2 && strcmp(argv[1], "-v") == 0) {
		printf(CopyrightVersion, Version);
		exit(0);
	}

	/*
	 * If "-cmd" arg is given, execute command and exit (o/w return).
	 */
	CmdLine(argc, argv);

	dsp = XtAppInitialize(&xtcontext, "XNetintf", options,
		XtNumber(options), &argc, argv, NULL, NULL, 0);

	if (argc != 1) {		/* display usage information */
		char spc[41];
		int i;

		if ((i = strlen(argv[0]) + 8) > 40) i = 40;
		memset(spc, ' ', i); spc[i] = '\0';
		fprintf(stderr, "usage: %s [Toolkit-Options]\n", argv[0]);
		fprintf(stderr, "%s[-ifname <interface>]\n", spc);
		fprintf(stderr, "%s[-confdir <config-directory>]\n", spc);
		fprintf(stderr, "%s[-maxival <seconds>]\n", spc);
		fprintf(stderr, "%s[-volume <percentage>]\n", spc);
		fprintf(stderr, "%s[-dbglvl <level>]\n", spc);
		exit(1);
	}

	/*
	 * Register f.delete to bring down interface and close display.
	 */
	XtAppAddActions(xtcontext, xnetintf_acts, XtNumber(xnetintf_acts));
	XtOverrideTranslations(dsp,
		XtParseTranslationTable("<Message>WM_PROTOCOLS: delwin()"));

	/*
	 * Create and realize our "netintf" widget, set up WM_DELETE_WINDOW
	 * property (f.delete) and enter the event loop.
	 */
	w_netintf = XtCreateManagedWidget("netintf", netintfWidgetClass,
		dsp, NULL, 0);
	XtRealizeWidget(dsp);
	wm_del = XInternAtom(XtDisplay(dsp), "WM_DELETE_WINDOW", False);
	(void) XSetWMProtocols(XtDisplay(dsp), XtWindow(dsp), &wm_del, 1);
	XtAppMainLoop(xtcontext);

	/*NOTREACHED*/
	exit(1);
}

static void
delwin(w, event, param, nparam)
	Widget w;
	XEvent *event;
	String *param;
	Cardinal *nparam;
{
	if (event->type == ClientMessage && event->xclient.data.l[0] != wm_del){
		XBell(XtDisplay(w), 0);
		return;
	}

	/*
	 * Bring down the interface, and then close the display.
	 */
	NetintfExit(w_netintf);
	XCloseDisplay(XtDisplay(w));
	exit(0);
}
